<?php
/**
*
* @package - phpbb3 UPS Easy Shop
* @version $Id: functions_shop.php 259 2010-08-28 09:30:32Z femu $
* @copyright (c) Wuerzi (http://spieleresidenz.de), (c) femu (http://die-muellers.org)
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/*
 * @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}


/**
* Set shop config value. Creates missing config entry.
*/
function set_shop_config($config_name, $config_value)
{
	global $db, $cache, $config, $shop_config;

	$sql = 'UPDATE ' . SHOP_CONFIG_TABLE . "
		SET config_value = '" . $db->sql_escape($config_value) . "'
		WHERE config_name = '" . $db->sql_escape($config_name) . "'";
	$db->sql_query($sql);

	if (!$db->sql_affectedrows() && !isset($shop_config[$config_name]))
	{
		$sql = 'INSERT INTO ' . SHOP_CONFIG_TABLE . ' ' . $db->sql_build_array('INSERT', array(
			'config_name'	=> $config_name,
			'config_value'	=> $config_value,
			));
		$db->sql_query($sql);
	}

	$shop_config[$config_name] = $config_value;
}

class item
{
	var $data;	//holds everything taken from phpbb_items (eg name, description, price, quantity)
	var $notrash = false; //set to true by buy function when instant_used so it isn't removed from inventory

	function item($db_info)
	{
		$this->data	= $db_info;
		// Here, we load custom config values
		$this->init();
	}

	function init()
	{
		// Blank by default
	}

	function get_actions()
	{
		global $user, $shop, $db, $config, $shop_config;

		$actions = array();
		if ($this->data['can_sellback'])
		{
			$actions['sellback'] = array(
				'name'		=> sprintf($user->lang['SHOP_ITEM_SELLBACK'], $shop->cash->format_cash_string($this->data['sell_price']), $config['points_name']),
				'confirm'	=> sprintf($user->lang['SHOP_ITEM_CONFIRM_SELL'], $this->data['name'], $shop->cash->format_cash_string($this->data['sell_price']), $config['points_name']),
				'function'	=> 'sellback'
			);
		}

		if ($this->data['can_trash'])
		{
			$actions['trash'] = array(
				'name'		=> $user->lang['SHOP_ITEM_TRASH'],
				'confirm'	=> sprintf($user->lang['SHOP_ITEM_CONFIRM_TRASH'], $this->data['name']),
				'function'	=> 'trash'
			);
		}

		if ($this->data['can_display'] && $config['shop_shelf_size_sig'] != 0)
		{
			// Find out how many items are currently on the shelf
			$sql = 'SELECT *
				FROM ' . SHOP_ITEMS_USER_TABLE . '
				WHERE user = ' . (int) $user->data['user_id'] . '
				AND shelf_display = true';
			$db->sql_query($sql);

			if ($this->data['shelf_display'])
			{
				$actions['display'] = array(
					'name'		=> $user->lang['SHOP_ITEM_DISPLAY_REMOVE'],
					'confirm'	=> false,
					'function'	=> 'display',
				);
			}
			elseif ($db->sql_affectedrows() == $config['shop_shelf_size_sig'])
			{
				$actions['display'] = array(
					'name'		=> $user->lang['SHOP_ITEM_DISPLAY_FULL'],
					'confirm'	=> false,
					'function'	=> 'none',
				);
			}
			else
			{
				$actions['display'] = array(
					'name'		=> $user->lang['SHOP_ITEM_DISPLAY'],
					'confirm'	=> false,
					'function'	=> 'display',
				);
			}
		}
		return $actions;
	}

	// This removes one instance of an item from the user's inventory.
	function remove_item()
	{
		global $db, $user, $shop, $phpbb_root_path, $phpEx;

		if ($this->notrash)
		{
			return false;
		}

		$sql = 'UPDATE ' . SHOP_ITEMS_USER_TABLE . '
			SET quantity_user = quantity_user - 1
			WHERE item=' . (int) $this->data['item'] . '
				AND user=' . (int) $user->data['user_id'];
		$db->sql_query($sql);

		$sql = 'DELETE FROM ' . SHOP_ITEMS_USER_TABLE . '
			WHERE quantity_user = 0';
		$db->sql_query($sql);
	}

	function action_handler($action, $confirm = true)
	{
		$actions = $this->get_actions();

		if (!isset($actions[$action]))
		{
			return false;
		}

		if (!$confirm or !$actions[$action]['confirm'] or confirm_box(true))
		{
			if ($actions[$action]['function'] == 'none')
			{
				return;
			}
			else
			{
				return $this->$actions[$action]['function'](); // Greatest single line of code I've ever written
			}
		}
		else
		{
			$s_hidden_fields = build_hidden_fields(array(
				'submit'	=> true,
				)
			);

			confirm_box(false, $actions[$action]['confirm'], $s_hidden_fields);
			return false;
		}

	}

	function display()
	{
		global $db, $user;

		if ($this->data['shelf_display'] == 1)
		{
			$this->data['shelf_display'] = 0;
		}
		else
		{
			$this->data['shelf_display'] = 1;
		}

		$sql = 'UPDATE ' . SHOP_ITEMS_USER_TABLE . '
			SET shelf_display=' . $this->data['shelf_display'] . '
			WHERE item=' . (int) $this->data['item'] . '
				AND user=' . (int) $user->data['user_id'];
		$db->sql_query($sql);

		return sprintf(($this->data['shelf_display'] == 1)?$user->lang['SHOP_ITEM_SHELF_ADDED']:$user->lang['SHOP_ITEM_SHELF_REMOVED'], $this->data['name']);
	}

	function trash()
	{
		global $db, $user, $shop, $phpbb_root_path, $phpEx, $shop_config;

		$this->remove_item();

		// Add Userlog
		if ($shop_config['shop_log_user_activity'] = 1)
		{
			add_log('user', $user->data['user_id'], 'LOG_SHOP_TRASH_ITEM', $this->data['name']);
		}

		return sprintf($user->lang['SHOP_ITEM_TRASHED'], $this->data['name']);
	}

	function sellback()
	{
		global $db, $user, $shop, $phpbb_root_path, $phpEx, $shop_config;

		// This part is exactly like throwing the item out
		$this->remove_item();
		// But then, they get points!!!
		$shop->cash->deposit_user($this->data['sell_price']);
		// And put it back in the shop
		$sql = 'UPDATE ' . SHOP_ITEMS_TABLE . '
			SET quantity = quantity + 1
			WHERE item=' . (int) $this->data['item'];
		$db->sql_query($sql);

		// Add Userlog
		if ($shop_config['shop_log_user_activity'] = 1)
		{
			add_log('user', $user->data['user_id'], 'LOG_SHOP_SELL_BACK', $this->data['name']);
		}

		return sprintf($user->lang['SHOP_ITEM_SOLD'], $this->data['name']);
	}

	// This function sees if the user is able to buy and item and charges the user if so
	// it doesn't move the item from the shop to the user's inventory
	// if item can't be bought, it won't return
	function prebuy()
	{
		global $user, $shop, $db, $phpEx, $phpbb_root_path, $config, $shop_config;

		// Eventually, we'll have an auth check here too.  for now, all registered users can use the shop
		if (!$user->data['is_registered'])
		{
			$db->sql_transaction('rollback');
			trigger_error('SHOP_NOT_PERMITTED');
		}

		if (!$this->data['quantity'])
		{
			$db->sql_transaction('rollback');
			$meta_info = append_sid("{$phpbb_root_path}shop.$phpEx", "mode=cat");
			meta_refresh(3, $meta_info);
			$message = $user->lang['SHOP_SOLD_OUT'] . '<br /><br />' . sprintf($user->lang['RETURN_PAGE'], '<a href="' . $meta_info . '">', '</a>');
			trigger_error($message);
		}

		// Check, if the user has more items, than allowed
		$sql = 'SELECT quantity_user
			FROM ' . SHOP_ITEMS_USER_TABLE . '
			WHERE item = ' . (int) $this->data['item'] . '
				AND user = ' . (int) $user->data['user_id'];
		$result = $db->sql_query($sql);
		$user_quantity = (int) $db->sql_fetchfield('quantity_user');
		$db->sql_freeresult($result);

		if ($this->data['max_per_user'] > 0 && $user_quantity >= $this->data['max_per_user'])
		{
			$meta_info = append_sid("{$phpbb_root_path}shop.$phpEx", "mode=cat");
			$message = sprintf($user->lang['SHOP_TOO_MUCH'], $user_quantity, $this->data['max_per_user']) . '<br /><br />' . sprintf($user->lang['SHOP_RETURN_SHOP'], '<a href="' . $meta_info . '">', '</a>');
			trigger_error($message);
		}

		// Charge the user
		if (!$shop->cash->charge_user($this->data['price']))
		{
			$db->sql_transaction('rollback');
			$meta_info = append_sid("{$phpbb_root_path}shop.$phpEx");
			meta_refresh(3, $meta_info);
			$message = sprintf($user->lang['SHOP_INSUFFICIENT_FUNDS'], $config['points_name']) . '<br /><br />' . sprintf($user->lang['RETURN_PAGE'], '<a href="' . $meta_info . '">', '</a>');
			trigger_error($message);
		}

		// Decrease the quantity in the shop
		$sql = 'UPDATE ' . SHOP_ITEMS_TABLE . '
			SET quantity = quantity - 1
			WHERE item = ' . (int) $this->data['item'];
		$db->sql_query($sql);

		// Check if the qty of the warehouse is below given value and refill, if enabled
		if ($this->data['auto_refill'])
		{
			// Check if min quantity is reached
			if ($this->data['quantity'] <= $this->data['auto_refill_min'] + 1)
			{
				$sql = 'UPDATE ' . SHOP_ITEMS_TABLE . '
					SET quantity = quantity + ' . (int) $this->data['auto_refill_qty'] . '
					WHERE item = ' . (int) $this->data['item'];
				$db->sql_query($sql);
			}
		}
	}

	function use_item()
	{
		// We only define this function so the program doesn't crash
		// if somebody decides to make an item that doesn't have a use action set
		// an instant use item
		return false;
	}

	function buy($cat_id)
	{
		global $template, $user, $shop, $db, $phpEx, $phpbb_root_path, $config, $shop_config;
		$db->sql_transaction('begin');
		$this->prebuy(); // Returns if purchase is allowed, user was charged, and quantity was decreased

		$sql1 = 'SELECT *
			FROM ' . SHOP_CONFIG_USER_TABLE . '
			WHERE user_id = ' . (int) $user->data['user_id'];
		$result1 = $db->sql_query($sql1);
		$row1 = $db->sql_fetchrow($result1);
		$db->sql_freeresult($result1);

		// Read out config data
		$sql = 'SELECT config_name, config_value
			FROM ' . SHOP_CONFIG_TABLE;
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$shop_config[$row['config_name']] = $row['config_value'];
		}
		$db->sql_freeresult($result);

		// Send PM to user if that option is on
		if ($shop_config['shop_pm_on_buy'] == 1 && $row1['shop_pm_user'] == 1)
		{
			require($phpbb_root_path . 'includes/functions_privmsgs.' . $phpEx);

			$my_subject = utf8_normalize_nfc($user->lang['SHOP_ITEM_PM_SUBJECT']);
			$my_text = utf8_normalize_nfc(sprintf($user->lang['SHOP_ITEM_PM_MESSAGE'], $user->data['username'], $this->data['name'], $shop->cash->format_cash_string($this->data['price']), $config['points_name'], $config['shop_name']));

			$pm_admin = $shop_config['shop_pm_admin'];

			$poll = $uid = $bitfield = $options = '';
			generate_text_for_storage($my_subject, $uid, $bitfield, $options, false, false, false);
			generate_text_for_storage($my_text, $uid, $bitfield, $options, true, true, true);

			$data = array(
			'address_list'		=> array ('u' => array($user->data['user_id'] => 'to')),
			'from_user_id' 		=> $pm_admin,
			'from_username' 	=> $user->data['username'],
			'icon_id'			=> 0,
			'from_user_ip'		=> '',

			'enable_bbcode' 	=> true,
			'enable_smilies' 	=> true,
			'enable_urls' 		=> true,
			'enable_sig' 		=> true,

			'message' 			=> $my_text,
			'bbcode_bitfield' 	=> $bitfield,
			'bbcode_uid' 		=> $uid,
			);

			submit_pm('post', $my_subject, $data, false);
		}

		if ($this->data['instant_use'])
		{
			$this->notrash = true;
			// This way, if it returns false, we still buy the item like normal
			if ($message = $this->use_item())
			{
				$meta_info = append_sid("{$phpbb_root_path}shop.$phpEx", "mode=showcat&amp;c=" . $cat_id);
				meta_refresh(3, $meta_info);
				$message = $message . '<br /><br />' . sprintf($user->lang['RETURN_PAGE'], '<a href="' . $meta_info . '">', '</a>');
				trigger_error($message);
			}
		}

		$sql = 'UPDATE ' . SHOP_ITEMS_USER_TABLE . '
			SET quantity_user = quantity_user + 1
			WHERE item = ' . (int) $this->data['item'] . '
				AND user = ' . (int) $user->data['user_id'];
		$db->sql_query($sql);

		if ($db->sql_affectedrows() != 1)
		{
			if ($row1['shop_shelf_display'] == 1)
			{
				$auto_display = 1;
			}
			else
			{
				$auto_display = 0;
			}

			$sql = 'INSERT INTO ' . SHOP_ITEMS_USER_TABLE . ' ' . $db->sql_build_array('INSERT', array(
				'item' => $this->data['item'],
				'user' => $user->data['user_id'],
				'quantity_user' => 1,
				'shelf_display'	=> $auto_display));
			$db->sql_query($sql);
		}
		$db->sql_transaction('commit');

		// Add Userlog
		if ($shop_config['shop_log_user_activity'] = 1)
		{
			add_log('user', $user->data['user_id'], 'LOG_SHOP_BUY_ITEM', $this->data['name']);
		}

		// Everything worked, send a success message
		$meta_info = append_sid("{$phpbb_root_path}shop.$phpEx", "mode=showcat&amp;c=" . $cat_id);
		meta_refresh(3, $meta_info);
		$message = sprintf($user->lang['SHOP_ITEM_BOUGHT'], $this->data['name']) . '<br /><br />' . sprintf($user->lang['RETURN_PAGE'], '<a href="' . $meta_info . '">', '</a>');
		trigger_error($message);
	}

	function buy_from_user($item_id, $user_id, $buyer, $coming_from)
	{
		global $template, $user, $shop, $db, $phpEx, $phpbb_root_path, $config, $shop_config;

		// Check if user is trying to buy from himself
		if ($user_id == $buyer)
		{
			$meta_info = append_sid("{$phpbb_root_path}shop.$phpEx", "mode=inventory_list&amp;f=2");
			$message = $user->lang['SHOP_BUY_REASON_YOURSELF'] . '<br /><br />' . sprintf($user->lang['RETURN_PAGE'], '<a href="' . $meta_info . '">', '</a>');
			trigger_error($message);
		}

		// Get item name
		$sql = 'SELECT name
			FROM ' . SHOP_ITEMS_TABLE . '
			WHERE item = ' . (int) $item_id;
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$item_name = $row['name'];
		$db->sql_freeresult($result);

		// Check, if buyer has enough points to buy
		$sql = 'SELECT sell_to_price
			FROM ' . SHOP_ITEMS_USER_TABLE . '
			WHERE user = ' . (int) $user_id . '
				AND item = ' . (int) $item_id;
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$sell_to_price = $row['sell_to_price'];
		$db->sql_freeresult($result);

		if ($sell_to_price > $user->data['user_points'])
		{
			$meta_info = append_sid("{$phpbb_root_path}memberlist.$phpEx", "mode=viewprofile&amp;u=" . $user_id);
			$message = sprintf($user->lang['SHOP_INSUFFICIENT_FUNDS'], $config['points_name']) . '<br /><br />' . sprintf($user->lang['RETURN_PAGE'], '<a href="' . $meta_info . '">', '</a>');
			trigger_error($message);
		}

		// Buyer has enough points, therefore let's buy the item
		if ($sell_to_price <= $user->data['user_points'])
		{
			// Read out needed config data from seller
			$sql1 = 'SELECT *
				FROM ' . SHOP_CONFIG_USER_TABLE . '
				WHERE user_id = ' . (int) $user_id;
			$result1 = $db->sql_query($sql1);
			$row1 = $db->sql_fetchrow($result1);
			$db->sql_freeresult($result1);

			// Read out needed config data from buyer
			$sql2 = 'SELECT *
				FROM ' . SHOP_CONFIG_USER_TABLE . '
				WHERE user_id = ' . (int) $user->data['user_id'];
			$result2 = $db->sql_query($sql2);
			$row2 = $db->sql_fetchrow($result2);
			$db->sql_freeresult($result1);

			// Read out sellers quantity
			$quantity_user = '';
			$sql3 = 'SELECT quantity_user
				FROM ' . SHOP_ITEMS_USER_TABLE . '
				WHERE user = ' . (int) $user_id . '
					AND item = ' . $item_id;
			$result3 = $db->sql_query($sql3);
			$row3 = $db->sql_fetchrow($result3);
			$quantity_user = $row3['quantity_user'];
			$db->sql_freeresult($result3);

			// Decrease the item from the user
			if ($quantity_user == 1)
			{
				$sql = 'DELETE FROM ' . SHOP_ITEMS_USER_TABLE . '
					WHERE item = ' . (int) $item_id . '
						AND user = ' . (int) $user_id;
				$db->sql_query($sql);
			}
			else
			{
				$sql = 'UPDATE ' . SHOP_ITEMS_USER_TABLE . '
					SET quantity_user = quantity_user - 1
					WHERE item = ' . (int) $item_id . '
						AND user = ' . (int) $user_id;
				$db->sql_query($sql);
			}

			// Charge the buyer
			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_points = user_points - ' . $sell_to_price . '
				WHERE user_id = ' . (int) $user->data['user_id'];
			$db->sql_query($sql);

			// Update or insert into buyers inventory
			$sql = 'SELECT item
				FROM ' . SHOP_ITEMS_USER_TABLE . '
				WHERE user = ' . (int) $user->data['user_id'] . '
					AND item = ' . (int) $item_id;
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if (!$row['item'])
			{
				if ($row2['shop_shelf_display'] == 1)
				{
					$auto_display = 1;
				}
				else
				{
					$auto_display = 0;
				}

				$sql = 'INSERT INTO ' . SHOP_ITEMS_USER_TABLE . ' ' . $db->sql_build_array('INSERT', array(
					'item' => $item_id,
					'user' => $user->data['user_id'],
					'quantity_user' => 1,
					'shelf_display'	=> $auto_display,
					'sell_to_price' => 0));
				$db->sql_query($sql);
			}
			else
			{
				$sql = 'UPDATE ' . SHOP_ITEMS_USER_TABLE . '
					SET quantity_user = quantity_user + 1
					WHERE item = ' . (int) $item_id . '
						AND user = ' . (int) $user->data['user_id'];
				$db->sql_query($sql);
			}

			// Give the points to the seller
			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_points = user_points + ' . $sell_to_price . '
				WHERE user_id = ' . (int) $user_id;
			$db->sql_query($sql);

			// Read out config data
			$sql = 'SELECT config_name, config_value
				FROM ' . SHOP_CONFIG_TABLE;
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				$shop_config[$row['config_name']] = $row['config_value'];
			}
			$db->sql_freeresult($result);

			// Read out the username from the seller
			$sql = 'SELECT username
				FROM ' . USERS_TABLE . '
				WHERE user_id = ' . $user_id;
			$result = $db->sql_query($sql);
			$username = $db->sql_fetchfield('username');
			$db->sql_freeresult($result1);

			// Add Userlog
			if ($shop_config['shop_log_user_activity'] = 1)
			{
				add_log('user', $user->data['user_id'], 'LOG_SHOP_BUY_ITEM_FROM', $this->data['name'], $username);
			}

			// Send PM to seller if that option is on
			if ($shop_config['shop_pm_on_buy'] == 1 && $row1['shop_pm_user'] == 1)
			{
				require($phpbb_root_path . 'includes/functions_privmsgs.' . $phpEx);

				$user_from_subject = utf8_normalize_nfc($user->lang['SHOP_PM_SUBJECT_FROM']);
				$user_from_text = utf8_normalize_nfc(sprintf($user->lang['SHOP_PM_MESSAGE_FROM'], $username, $user->data['username'], $item_name, $shop->cash->format_cash_string($sell_to_price), $config['points_name'], $config['shop_name']));

				$poll = $uid = $bitfield = $options = '';
				generate_text_for_storage($user_from_subject, $uid, $bitfield, $options, false, false, false);
				generate_text_for_storage($user_from_text, $uid, $bitfield, $options, true, true, true);

				$data = array(
				'address_list'		=> array ('u' => array($user_id => 'to')),
				'from_user_id' 		=> $user->data['user_id'],
				'from_username' 	=> $user->data['username'],
				'icon_id'			=> 0,
				'from_user_ip'		=> '',

				'enable_bbcode' 	=> true,
				'enable_smilies' 	=> true,
				'enable_urls' 		=> true,
				'enable_sig' 		=> true,

				'message' 			=> $user_from_text,
				'bbcode_bitfield' 	=> $bitfield,
				'bbcode_uid' 		=> $uid,
				);

				submit_pm('post', $user_from_subject, $data, false);
			}

			// Send PM to buyer if that option is on
			if ($shop_config['shop_pm_on_buy'] == 1 && $row2['shop_pm_user'] == 1)
			{
				//require($phpbb_root_path . 'includes/functions_privmsgs.' . $phpEx);

				$user_from_subject = utf8_normalize_nfc($user->lang['SHOP_PM_SUBJECT_TO']);
				$user_from_text = utf8_normalize_nfc(sprintf($user->lang['SHOP_PM_MESSAGE_TO'], $user->data['username'], $username, $item_name, $shop->cash->format_cash_string($sell_to_price), $config['points_name'], $config['shop_name']));

				$poll = $uid = $bitfield = $options = '';
				generate_text_for_storage($user_from_subject, $uid, $bitfield, $options, false, false, false);
				generate_text_for_storage($user_from_text, $uid, $bitfield, $options, true, true, true);

				$data = array(
				'address_list'		=> array ('u' => array($user->data['user_id'] => 'to')),
				'from_user_id' 		=> $user->data['user_id'],
				'from_username' 	=> $user->data['username'],
				'icon_id'			=> 0,
				'from_user_ip'		=> '',

				'enable_bbcode' 	=> true,
				'enable_smilies' 	=> true,
				'enable_urls' 		=> true,
				'enable_sig' 		=> true,

				'message' 			=> $user_from_text,
				'bbcode_bitfield' 	=> $bitfield,
				'bbcode_uid' 		=> $uid,
				);

				submit_pm('post', $user_from_subject, $data, false);
			}

			// Everything worked, send a success message
			$meta_info = ($coming_from == 1) ? append_sid("{$phpbb_root_path}memberlist.$phpEx", "mode=viewprofile&amp;u=" . $user_id) :  append_sid("{$phpbb_root_path}shop.$phpEx", "mode=inventory_list");
			$message = sprintf($user->lang['SHOP_ITEM_BOUGHT'], $item_name) . '<br /><br />' . sprintf($user->lang['RETURN_PAGE'], '<a href="' . $meta_info . '">', '</a>');
			trigger_error($message);
		}
	}
}

class shop
{
	var $cash;
	// Constructor
	function shop()
	{
		global $config;

		// Select money system
		$this->cash = new ups_points;
	}

	function get_item_by_row($row)
	{
		global $phpbb_root_path, $phpEx;

		include_once($phpbb_root_path . 'includes/shop/items/' . $row['backend'] . '.' . $phpEx);

		return new $row['backend']($row);
	}

	function get_item_by_row_inventory($row)
	{
		global $phpbb_root_path, $phpEx;

		include_once($phpbb_root_path . 'includes/shop/items/' . $row['backend'] . '.' . $phpEx);

		return new $row['backend']($row);
	}

	function get_item($item_id, $condition = false)
	{
		global $db;

		$condition = $condition?' && ' . $condition:'';

		$sql = 'SELECT *
			FROM ' . SHOP_ITEMS_TABLE . '
			WHERE item=' . (int) $item_id . $condition;
		$result = $db->sql_query($sql);

		if ($db->sql_affectedrows() != 1)
		{
			return false;
		}
		return 	$this->get_item_by_row($db->sql_fetchrow($result));
	}

	function get_item_inventory($item_id, $user_id, $condition = false)
	{
		global $db;

		$condition = $condition?' && ' . $condition:'';

		$sql = 'SELECT *
			FROM ' . SHOP_ITEMS_USER_TABLE . '
			WHERE item = ' . (int) $item_id . $condition . '
				AND user = '. (int) $user_id;
		$result = $db->sql_query($sql);

		if ($db->sql_affectedrows() != 1)
		{
			return false;
		}
		return 	$this->get_item_by_row($db->sql_fetchrow($result));
	}

	function get_item_list($condition = false)
	{
		global $db, $shop_config;

		// Read out config data
		$sql = 'SELECT config_name, config_value
			FROM ' . SHOP_CONFIG_TABLE;
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$shop_config[$row['config_name']] = $row['config_value'];
		}
		$db->sql_freeresult($result);

		$start 		= request_var('start', 0);
		$cat		= request_var('cat', 1);
		$number		= $shop_config['items_per_page'];
		$condition 	= $condition?' WHERE ' . $condition:'';

		$this->generate_cat_nav($cat);

		$sql = 'SELECT i.*, c.*
			FROM ' . SHOP_ITEMS_TABLE . $condition . ' i
			LEFT JOIN ' . SHOP_CATS_TABLE . ' c
				ON c.cat_id = i.shop_cat_id
			WHERE i.shop_cat_id = "' . (int) $cat . '"
			ORDER BY i.item_order';
		$result = $db->sql_query_limit($sql, $number, $start);

		$items = array();

		if ($db->sql_affectedrows() != 0)
		{
			while($item_row = $db->sql_fetchrow($result))
			{
				$items[] = $this->get_item_by_row($item_row);
			}
			return $items;
		}
		else
		{
			return false;
		}
	}

	function get_item_list_acp($condition = false)
	{
		global $db, $shop_config;

		// Read out config data
		$sql = 'SELECT config_name, config_value
			FROM ' . SHOP_CONFIG_TABLE;
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$shop_config[$row['config_name']] = $row['config_value'];
		}
		$db->sql_freeresult($result);

		$start 		= request_var('start', 0);
		$number		= $shop_config['items_per_page'];
		$condition 	= $condition?' WHERE ' . $condition:'';

		$this->generate_cat_nav($cat);

		$sql = 'SELECT *
			FROM ' . SHOP_ITEMS_TABLE . '
			ORDER BY item_order';
		$result = $db->sql_query_limit($sql, $number, $start);

		$items = array();

		if ($db->sql_affectedrows() != 0)
		{
			while($item_row = $db->sql_fetchrow($result))
			{
				$items[] = $this->get_item_by_row($item_row);
			}
			return $items;
		}
		else
		{
			return false;
		}
	}

	function get_item_user($item_id, $user_id = NULL,  $condition = false)
	{
		global $db, $user;

		$user_id = is_null($user_id) ? $user->data['user_id'] : $user_id;
		$condition = $condition?' && ' . $condition:'';

		$sql = 'SELECT *
			FROM ( ' . SHOP_ITEMS_TABLE . '
			INNER JOIN ' . SHOP_ITEMS_USER_TABLE . '
				ON ' . SHOP_ITEMS_USER_TABLE . '.item = ' . SHOP_ITEMS_TABLE . '.item)
			WHERE user = ' . (int) $user_id . '
				AND item_user=' . (int) $item_id . $condition;
		$result = $db->sql_query($sql);

		if ($db->sql_affectedrows() != 1)
		{
			return false;
		}
		return $this->get_item_by_row($db->sql_fetchrow($result));
	}

	function get_item_list_ucp($user_id = NULL, $condition = false)
	{
		global $db, $user;

		// Read out config data
		$sql = 'SELECT config_name, config_value
			FROM ' . SHOP_CONFIG_TABLE;
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$shop_config[$row['config_name']] = $row['config_value'];
		}
		$db->sql_freeresult($result);

		$start 		= request_var('start', 0);
		$number		= $shop_config['items_per_page'];

		$user_id = is_null($user_id) ? $user->data['user_id'] : $user_id;
		$condition = $condition?' && ' . $condition:'';

		$sql = 'SELECT *
			FROM ( ' . SHOP_ITEMS_TABLE . '
			INNER JOIN ' . SHOP_ITEMS_USER_TABLE . '
				ON ' . SHOP_ITEMS_USER_TABLE . '.item = ' . SHOP_ITEMS_TABLE . '.item)
			WHERE user = ' . (int) $user_id . $condition . '
			ORDER BY item_order';
		$result = $db->sql_query_limit($sql, $number, $start);

		$items = array();

		if ($db->sql_affectedrows() != 0)
		{
			while ($item_row = $db->sql_fetchrow($result))
			{
				$items[] = $this->get_item_by_row($item_row);
			}
			return $items;
		}
		else
		{
			return false;
		}
	}

	function get_item_list_user($user_id = NULL, $condition = false)
	{
		global $db, $user;

		$user_id = is_null($user_id) ? $user->data['user_id'] : $user_id;
		$condition = $condition?' && ' . $condition:'';

		$sql = 'SELECT *
			FROM ( ' . SHOP_ITEMS_TABLE . '
			INNER JOIN ' . SHOP_ITEMS_USER_TABLE . '
				ON ' . SHOP_ITEMS_USER_TABLE . '.item = ' . SHOP_ITEMS_TABLE . '.item)
			WHERE user = ' . (int) $user_id . $condition . '
			ORDER BY item_order';
		$result = $db->sql_query($sql);

		$items = array();

		if ($db->sql_affectedrows() != 0)
		{
			while ($item_row = $db->sql_fetchrow($result))
			{
				$items[] = $this->get_item_by_row($item_row);
			}
			return $items;
		}
		else
		{
			return false;
		}
	}

	// Generate the navigation bar
	function generate_cat_nav(&$cat_data)
	{
		global $db, $user, $template, $auth;
		global $phpEx, $phpbb_root_path;

		// Get video parents
		$cat_parents = $this->get_cat_parents($cat_data);

		// Build navigation links
		if (!empty($cat_parents))
		{
			foreach ($cat_parents as $parent_cat_id => $parent_data)
			{
				list($parent_name, $parent_type) = array_values($parent_data);
				$template->assign_block_vars('navlinks', array(
					'FORUM_NAME'	=> $parent_name,
					'U_VIEW_FORUM'	=> append_sid("{$phpbb_root_path}shop.$phpEx", "mode=showcat&amp;c=" . $parent_cat_id),
				));
			}
		}

		$template->assign_block_vars('navlinks', array(
			'FORUM_NAME'	=> $cat_data['cat_name'],
			'U_VIEW_FORUM'	=> append_sid("{$phpbb_root_path}shop.$phpEx", "mode=showcat&amp;c=" . $cat_data['cat_id']),
		));
		return;
	}

	// Returns cat parents as an array
	function get_cat_parents(&$cat_data)
	{
		global $db;

		$cat_parents = array();
		if ($cat_data['parent_id'] > 0)
		{
			if ($cat_data['cat_parents'] == '')
			{
				$sql = 'SELECT cat_id, cat_name
					FROM ' . SHOP_CATS_TABLE . '
					WHERE left_id < ' . (int) $cat_data['left_id'] . '
						AND right_id > ' . (int) $cat_data['right_id'] . '
					ORDER BY left_id ASC';
				$result = $db->sql_query($sql);

				while ($row = $db->sql_fetchrow($result))
				{
					$row['cat_type'] = 1;
					$cat_parents[$row['cat_id']] = array($row['cat_name'], (int) $row['cat_type']);
				}
				$db->sql_freeresult($result);

				$cat_data['cat_parents'] = serialize($cat_parents);

				$sql = 'UPDATE ' . SHOP_CATS_TABLE . "
					SET cat_parents = '" . $db->sql_escape($cat_data['cat_parents']) . "'
					WHERE parent_id = " . (int) $cat_data['parent_id'];
				$db->sql_query($sql);
			}
			else
			{
				$cat_parents = unserialize($cat_data['cat_parents']);
			}
		}
		return $cat_parents;
	}

	// Get the categories infos
	function get_cat_info($cat_id)
	{
		global $db;

		$sql = 'SELECT *
			FROM ' . SHOP_CATS_TABLE . '
			WHERE cat_id = ' . (int) $cat_id;
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$cat_data = $row;
		}
		return $cat_data;
	}

	// Generate the sub categories list
	function generate_cat_list($cat_id)
	{
		global $db, $template, $phpbb_root_path, $phpEx, $user, $config;

		$sql = 'SELECT bc.*, bd.item, COUNT(bd.item) AS number_items
			FROM ' . SHOP_CATS_TABLE . ' bc
			LEFT JOIN ' . SHOP_CATS_TABLE . ' bc2
				ON ( bc2.left_id < bc.right_id
					AND bc2.left_id > bc.left_id )
			LEFT JOIN ' . SHOP_ITEMS_TABLE . ' bd
				ON ( bd.shop_cat_id = bc.cat_id
					OR bd.shop_cat_id = bc2.cat_id )
			WHERE bc.parent_id = ' . (int) $cat_id . '
			GROUP BY bc.cat_id
			ORDER BY bc.left_id ASC';
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$number_items = 0;
			$subcats = '';

			// Check subs and number items within
			if (($row['left_id'] + 1) != $row['right_id'])
			{
				$sql2 = 'SELECT bc.*
					FROM ' . SHOP_CATS_TABLE . ' bc
					WHERE bc.parent_id = ' . $row['cat_id'] . '
					ORDER BY bc.left_id ASC';
				$result2 = $db->sql_query($sql2);

				while ($row2 = $db->sql_fetchrow($result2))
				{
					$subcats .= ($subcats) ? ', ' : '';
					$subcats .= '<a class="subforum ' . (((isset($read_info[$row2['cat_id']]) ? $read_info[$row2['cat_id']] : 0) && ($user->data['user_id'] != ANONYMOUS)) ? 'unread' : 'read') . '" href="';
					$subcats .= append_sid("{$phpbb_root_path}shop.$phpEx", "mode=showcat&amp;c=" . $row2['cat_id']);
					$subcats .= '">' . censor_text($row2['cat_name']) . '</a>';
				}
				$db->sql_freeresult($result2);

				$folder_image = 'forum_read_subforum';
				$folder_alt = 'no';
				$l_subcats = $user->lang['SHOP_SUB_CAT'];
				if ($row['left_id'] + 3 != $row['right_id'])
				{
					$l_subcats = $user->lang['SHOP_SUB_CATS'];
				}
			}
			else
			{
				$folder_image = 'forum_read';
				$l_subcats = '';
				$folder_alt = 'no';
			}

			if ($row['item'] != 0)
			{
				$sql2 = 'SELECT bd.item
					FROM ' . SHOP_ITEMS_TABLE . ' bd
					LEFT JOIN ' . SHOP_CATS_TABLE . ' bc
						ON bc.left_id >= ' . $row['left_id'] . '
							AND bc.right_id <= ' . $row['right_id'] . '
					WHERE bd.shop_cat_id = bc.cat_id';
				$result2 = $db->sql_query($sql2);
				while ($row2 = $db->sql_fetchrow($result2))
				{
					$number_items++;
				}
				$db->sql_freeresult($result2);
			}

			$cat_id = $row['cat_id'];

			// Send the results to the template
			$template->assign_block_vars('catrow', array(
				'SHOP_CAT_NAME'			=> censor_text($row['cat_name']),
				'NUMBER_ITEMS'			=> $number_items,
				'U_SHOP_CAT'			=> append_sid("{$phpbb_root_path}shop.$phpEx", "mode=showcat&amp;c=" . $row['cat_id']),
				'SHOP_CAT_DESC'			=> generate_text_for_display((string) $row['cat_desc'], (string) $row['cat_desc_uid'], (string) $row['cat_desc_bitfield'], (string) $row['cat_desc_options']),
				'CAT_FOLDER_IMG_SRC'	=> $user->img($folder_image, $folder_alt, false, '', 'src'),
				'SUBCATS'				=> ($subcats) ? $l_subcats . ': <span style="font-weight: bold;">' . $subcats . '</span>' : '',
			));
		}

		$db->sql_freeresult($result);

		$template->assign_vars(array(
			'LAST_POST_IMG'	=> $user->img('icon_topic_latest', 'VIEW_LATEST_POST'),
		));
		return $cat_id;
	}
}


class ups_points
{
	// Constructor
	function ups_points()
	{
		global $phpbb_root_path, $phpEx;

		if (!class_exists('ups_points'))
		{
			require($phpbb_root_path . 'includes/points/functions_points.' . $phpEx);
		}
		$this->money_on = true;
	}

	function format_cash_string($amount = NULL)
	{
		$amount = is_null($amount) ? $this->user_get_cash_amount() : $amount;
		return number_format_points($amount);
	}

	function deposit_user($amount, $user_id = NULL)
	{
		global $user;

		$user_id = is_null($user_id) ? $user->data['user_id'] : $user_id;
		add_points($user_id, $amount);
		return true;
	}

	function charge_user($amount, $user_id = NULL)
	{
		global $user;

		$user_id = is_null($user_id) ? $user->data['user_id'] : $user_id;

		if ($amount > $user->data['user_points'])
		{
			return false;
		}
		substract_points($user_id, $amount);
		return true;
	}

	function user_get_cash_amount($user_id = NULL)
	{
		global $user;

		$user_id = is_null($user_id) ? $user->data['user_id'] : $user_id;
		return $user->data['user_points'];
	}
}

?>
