<?php
/**
*
* @package - phpbb3 UPS Easy Shop
* @version $Id: install_shop.php 252 2010-08-24 17:11:14Z Wuerzi $
* @copyright (c) Wuerzi (http://spieleresidenz.de), (c) femu (http://die-muellers.org)
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
define('UMIL_AUTO', true);
define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup();

if (!file_exists($phpbb_root_path . 'umil/umil_auto.' . $phpEx))
{
	trigger_error('Please download the latest UMIL (Unified MOD Install Library) from: <a href="http://www.phpbb.com/mods/umil/">phpBB.com/mods/umil</a>', E_USER_ERROR);
}

// The name of the mod to be displayed during installation.
$mod_name = 'ACP_SHOP';

/*
* The name of the config variable which will hold the currently installed version
* You do not need to set this yourself, UMIL will handle setting and updating the version itself.
*/

$version_config_name = 'shop_mod_version';

/*
* The language file which will be included when installing
* Language entries that should exist in the language file for UMIL (replace $mod_name with the mod's name you set to $mod_name above)
*/
$language_file = 'mods/info_acp_shop';

/*
* Optionally we may specify our own logo image to show in the upper corner instead of the default logo.
* $phpbb_root_path will get prepended to the path specified
* Image height should be 50px to prevent cut-off or stretching.
*/

/*
* The array of versions and actions within each.
* You do not need to order it a specific way (it will be sorted automatically), however, you must enter every version, even if no actions are done for it.
*
* You must use correct version numbering.  Unless you know exactly what you can use, only use X.X.X (replacing X with an integer).
* The version numbering must otherwise be compatible with the version_compare function - http://php.net/manual/en/function.version-compare.php
*/

/**
* Define the basic structure
* The format:
*		array('{TABLE_NAME}' => {TABLE_DATA})
*		{TABLE_DATA}:
*			COLUMNS = array({column_name} = array({column_type}, {default}, {auto_increment}))
*			PRIMARY_KEY = {column_name(s)}
*			KEYS = array({key_name} = array({key_type}, {column_name(s)})),
*
*	Column Types:
*	INT:x		=> SIGNED int(x)
*	BINT		=> BIGINT
*	UINT		=> mediumint(8) UNSIGNED
*	UINT:x		=> int(x) UNSIGNED
*	TINT:x		=> tinyint(x)
*	USINT		=> smallint(4) UNSIGNED (for _order columns)
*	BOOL		=> tinyint(1) UNSIGNED
*	VCHAR		=> varchar(255)
*	CHAR:x		=> char(x)
*	XSTEXT_UNI	=> text for storing 100 characters (topic_title for example)
*	STEXT_UNI	=> text for storing 255 characters (normal input field with a max of 255 single-byte chars) - same as VCHAR_UNI
*	TEXT_UNI	=> text for storing 3000 characters (short text, descriptions, comments, etc.)
*	MTEXT_UNI	=> mediumtext (post text, large text)
*	VCHAR:x		=> varchar(x)
*	TIMESTAMP	=> int(11) UNSIGNED
*	DECIMAL		=> decimal number (5,2)
*	DECIMAL:	=> decimal number (x,2)
*	PDECIMAL	=> precision decimal number (6,3)
*	PDECIMAL:	=> precision decimal number (x,3)
*	VCHAR_UNI	=> varchar(255) BINARY
*	VCHAR_CI	=> varchar_ci for postgresql, others VCHAR
*/

$versions = array(
	// Version 0.5.0 - this is the first version using UMIL
	'0.5.0'	=> array(

		// Now to add some permission settings
		'permission_add' => array(
			array('u_shop_mod', true),
			array('a_shop_mod', true),
		),

		// How about we give some default permissions then as well?
		'permission_set' => array(
			array('REGISTERED', 'u_shop_mod', 'group'),
			array('ROLE_ADMIN_STANDARD', 'a_shop_mod', 'role'),
		),


		// Now to add the tables (this uses the layout from develop/create_schema_files.php and from phpbb_db_tools)
		'table_add' => array(

			array($table_prefix . 'shop_config', array(
					'COLUMNS'		=> array(
						'config_name'		=> array('VCHAR', ''),
						'config_value'		=> array('VCHAR_UNI', ''),
					),
					'PRIMARY_KEY'	=> 'config_name',
				),
			),

			array($table_prefix . 'shop_items_user', array(
					'COLUMNS'		=> array(
						'item_user'			=> array('UINT', NULL, 'auto_increment'),
						'item'				=> array('UINT', NULL, ''),
						'user'				=> array('UINT', NULL, ''),
						'quantity_user'		=> array('UINT', NULL, ''),
						'shelf_display'		=> array('TINT:1', 0),
					),
					'PRIMARY_KEY'	=> 'item_user',
				),
			),

			array($table_prefix . 'shop_items', array(
					'COLUMNS'		=> array(
						'item'							=> array('UINT', NULL, 'auto_increment'),
						'name'							=> array('VCHAR:100', ''),
						'description'					=> array('VCHAR', ''),
						'backend'						=> array('VCHAR', ''),
						'icon'							=> array('VCHAR:100', ''),
						'icon_width'					=> array('TINT:4', 0),
						'icon_height'					=> array('TINT:4', 0),
						'quantity'						=> array('UINT', 0),
						'price'							=> array('DECIMAL:20', 5.00),
						'sell_price'					=> array('DECIMAL:20', 3.00),
						'item_order'					=> array('UINT', 0),
						'instant_use'					=> array('TINT:1', 0),
						'can_sellback'					=> array('TINT:1', 0),
						'can_trash'						=> array('TINT:1', 0),
						'can_display'					=> array('TINT:1', 0),
					),
					'PRIMARY_KEY'	=> 'item',
				),
			),
		),

		// Clear the general cache as well as the templates, imagesets and themes cache
		'cache_purge' => array(
			array(),
			array('template'),
			array('theme'),
		),

		// Alright, now lets add some modules to the ACP
		'module_add' => array(
			// First, lets add a new category
			array('acp', 'ACP_CAT_DOT_MODS', 'ACP_SHOP'),

			array('acp', 'ACP_SHOP', array(
					'module_basename'	=> 'shop_mod',
					'module_langname'	=> 'ACP_SHOP_CONFIG',
					'module_mode'		=> 'config',
					'module_auth'		=> 'acl_a_shop_mod',
				),
			),

			array('acp', 'ACP_SHOP', array(
					'module_basename'	=> 'shop_mod',
					'module_langname'	=> 'ACP_SHOP_ITEMS',
					'module_mode'		=> 'items',
					'module_auth'		=> 'acl_a_shop_mod',
				),
			),

			array('ucp', '', 'UCP_SHOP'),

			array('ucp', 'UCP_SHOP', array(
					'module_basename'	=> 'shop',
					'module_langname'	=> 'UCP_SHOP_INVENTORY',
					'module_mode'		=> 'inventory',
					'module_auth'		=> 'acl_u_shop_mod',
				),
			),
		),

		/*
		* Now we need to insert some data.  The easiest way to do that is through a custom function
		* Enter 'custom' for the array key and the name of the function for the value.
		*/
		'custom'	=> 'first_fill_0_5_0',
	),

	// Version 0.6.0
	'0.6.0' => array(
		'module_add' => array(
			// Now we will add the forum points modlue to the ACP_POINTS category
			array('ucp', 'UCP_SHOP', array(
					'module_basename'	=> 'shop',
					'module_langname'	=> 'UCP_SHOP_PRESENT',
					'module_mode'		=> 'present',
					'module_auth'		=> 'acl_u_shop_mod',
				),
			),
		),

		'table_column_update' => array(
			array(SHOP_ITEMS_TABLE, 'icon_width', array('INT:5', 0)),
			array(SHOP_ITEMS_TABLE, 'icon_height', array('INT:5', 0)),
		),

		'custom'	=> 'fill_0_6_0',
	),

	// Version 0.7.0
	'0.7.0'	=> array(
		'table_add' => array(
			array($table_prefix . 'shop_cat', array(
					'COLUMNS'		=> array(
						'cat_id'			=> array('UINT', NULL, 'auto_increment'),
						'parent_id'			=> array('UINT', 0),
						'left_id'			=> array('UINT', 1),
						'right_id'			=> array('UINT', 2),
						'cat_parents'		=> array('MTEXT_UNI', ''),
						'cat_desc'			=> array('MTEXT_UNI', ''),
						'cat_desc_uid'		=> array('VCHAR:8', ''),
						'cat_desc_bitfield'	=> array('VCHAR', ''),
						'cat_desc_options'	=> array('UINT:3', 7),
						'cat_name'			=> array('VCHAR', ''),
					),
					'PRIMARY_KEY'	=> 'cat_id',
				),
			),

			array($table_prefix . 'shop_config_user', array(
					'COLUMNS'		=> array(
						'user_id'				=> array('UINT', NULL),
						'shop_pm_user'			=> array('TINT:1', 1),
						'shop_shelf_display'	=> array('TINT:1', 0),
					),
					'PRIMARY_KEY'	=> 'user_id',
				),
			),
		),

		'module_add' => array(
			// Add a new category
			array('acp', 'ACP_SHOP', array(
					'module_basename'	=> 'shop_mod',
					'module_langname'	=> 'ACP_SHOP_CATEGORIES',
					'module_mode'		=> 'manage_categories',
					'module_auth'		=> 'acl_a_shop_mod',
				),
			),

			// Now we will add the UCP selling panel
			array('ucp', 'UCP_SHOP', array(
					'module_basename'	=> 'shop',
					'module_langname'	=> 'UCP_SHOP_SELL',
					'module_mode'		=> 'sell',
					'module_auth'		=> 'acl_u_shop_mod',
				),
			),

			// Now we will add the UCP config panel
			array('ucp', 'UCP_SHOP', array(
					'module_basename'	=> 'shop',
					'module_langname'	=> 'UCP_SHOP_CONFIG',
					'module_mode'		=> 'config',
					'module_auth'		=> 'acl_u_shop_mod',
				),
			),
		),

		'custom'	=> 'fill_0_7_0',
	),
);

// Include the UMIF Auto file and everything else will be handled automatically.
include($phpbb_root_path . 'umil/umil_auto.' . $phpEx);

/*
* Here is our custom function that will be called for version 0.5.0
*
* @param string $action The action (install|update|uninstall) will be sent through this.
* @param string $version The version this is being run for will be sent through this.
*/
function first_fill_0_5_0($action, $version)
{
	global $db, $table_prefix, $umil;

	switch ($action)
	{
		case 'install' :
			// Run this when installing the first time
			if ($umil->table_exists($table_prefix . 'config'))
			{
				$sql_ary = array();

				$sql_ary[] = array('config_name' => 'shop_enable',			'config_value' => '1',);
				$sql_ary[] = array('config_name' => 'shop_shelf_size_sig',	'config_value' => '5',);
				$sql_ary[] = array('config_name' => 'shop_mod_version',		'config_value' => '0.5.0',);
				$sql_ary[] = array('config_name' => 'shop_name',			'config_value' => 'Forumshop',);

				$db->sql_multi_insert($table_prefix . 'config ', $sql_ary);
			}

			if ($umil->table_exists($table_prefix . 'shop_config'))
			{
				// before we fill anything in this table, we truncate it. Maybe someone missed an old installation.
				$db->sql_query('TRUNCATE TABLE ' . $table_prefix . 'shop_config');

				$sql_ary = array();

				$sql_ary[] = array('config_name' => 'shop_pm_on_buy',			'config_value' => '1',);
				$sql_ary[] = array('config_name' => 'shop_pm_admin',			'config_value' => '2',);
				$sql_ary[] = array('config_name' => 'items_per_page',			'config_value' => '10',);
				$sql_ary[] = array('config_name' => 'shop_items_viewtopic',		'config_value' => '1',);
				$sql_ary[] = array('config_name' => 'shop_items_profile',		'config_value' => '1',);

				$db->sql_multi_insert($table_prefix . 'shop_config ', $sql_ary);
			}

			// Send the message, that the command was successful
			return 'UMIL_SHOP_FIRST_FILL';
		break;

		case 'update' :
		break;

 		case 'uninstall' :
			// Run this additionally when uninstalling
			if ($umil->table_exists($table_prefix . 'config'))
			{
				$sql = 'DELETE FROM ' . $table_prefix . "config
					WHERE config_name = 'shop_enable'";
				$db->sql_query($sql);

				$sql = 'DELETE FROM ' . $table_prefix . "config
					WHERE config_name = 'shop_shelf_size_sig'";
				$db->sql_query($sql);

				$sql = 'DELETE FROM ' . $table_prefix . "config
					WHERE config_name = 'shop_name'";
				$db->sql_query($sql);
			}

			// Send the message, that the command was successful
			return 'UMIL_SHOP_REMOVE_ENTRIES';
		break;
	}
}

function fill_0_6_0($action, $version)
{
	global $db, $table_prefix, $umil;

	switch ($action)
	{
		case 'install' :

		case 'update' :
			if ($umil->table_exists($table_prefix . 'shop_config'))
			{
				$sql_ary = array();

				$sql_ary[] = array('config_name' => 'disable_message',			'config_value' => 'The Shop is currently disabled!',);

				$db->sql_multi_insert($table_prefix . 'shop_config ', $sql_ary);
			}

			// Send the message, that the command was successful
			return 'UMIL_SHOP_TABLE_UPDATED';
		break;
	}
}

function fill_0_7_0($action, $version)
{
	global $db, $table_prefix, $umil;

	switch ($action)
	{
		case 'install' :
		case 'update' :
			if ($umil->table_exists($table_prefix . 'shop_items'))
			{
				$sql = "ALTER TABLE " . $table_prefix . "shop_items
					ADD shop_cat_id MEDIUMINT(8) NOT NULL DEFAULT '0'";
				$db->sql_query($sql);

				$sql = "ALTER TABLE " . $table_prefix . "shop_items
					ADD auto_refill TINYINT(1) NOT NULL DEFAULT '0',
					ADD auto_refill_min TINYINT(3) NOT NULL DEFAULT '0',
					ADD auto_refill_qty INT NOT NULL DEFAULT '0',
					ADD max_per_user INT NOT NULL DEFAULT '0'";
				$db->sql_query($sql);
			}

			if ($umil->table_exists($table_prefix . 'shop_items_user'))
			{
				$sql = "ALTER TABLE " . $table_prefix . "shop_items_user
					ADD sell_to_price DECIMAL(11, 2) NOT NULL DEFAULT '0'";
				$db->sql_query($sql);
			}

			// Create the first category, which is needed to get the categories working
			if ($umil->table_exists($table_prefix . 'shop_cat'))
			{
				$sql_ary = array();
				$sql_ary[] = array('cat_id' => NULL, 'parent_id' => '0', 'left_id' => '1', 'right_id' => '2', 'cat_parents' => '', 'cat_desc' => 'This is a sample description. DO NOT DELETE THIS CATEGORY! Rename it and change the description!', 'cat_desc_uid' => '', 'cat_desc_bitfield' => '', 'cat_desc_options' => '7', 'cat_name' => 'Your First Categroy',);
				$db->sql_multi_insert($table_prefix . 'shop_cat ', $sql_ary);
			}

			// Put all items in the first category
			if ($umil->table_exists($table_prefix . 'shop_items'))
			{
				$sql = "UPDATE " . $table_prefix . "shop_items
					SET shop_cat_id = '1'";
				$db->sql_query($sql);
			}

			if ($umil->table_exists($table_prefix . 'shop_config'))
			{
				$sql_ary = array();

				$sql_ary[] = array('config_name' => 'shop_pm_admin_copy',		'config_value' => '0',);
				$sql_ary[] = array('config_name' => 'shop_pm_admin_warehouse',	'config_value' => '1',);
				$sql_ary[] = array('config_name' => 'shop_icon_height',			'config_value' => '50',);
				$sql_ary[] = array('config_name' => 'shop_icon_width',			'config_value' => '50',);
				$sql_ary[] = array('config_name' => 'shop_log_user_activity',	'config_value' => '1',);

				$db->sql_multi_insert($table_prefix . 'shop_config ', $sql_ary);
			}

			// Send the message, that the command was successful
			return 'UMIL_SHOP_TABLE_UPDATED';
		break;
	}
}

?>