<?php
/**
*
* @package - phpBB3 UPS Easy Shop
* @version $Id: shop.php 247 2010-08-24 16:16:48Z femu $
* @copyright (c) 2010 Wuerzi - http://spieleresidenz.de and femu - http://die-muellers.org
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);
include($phpbb_root_path . 'includes/shop/functions_shop.' . $phpEx);

// Create shop object
$shop = new shop;

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup(array('mods/shop', 'mods/points'));

// Load shop template
$template->set_filenames(array(
	'body' => 'shop.html')
);

//Template variables for the navigation
$template->assign_block_vars('navlinks', array(
	'FORUM_NAME'	=> $user->lang['SHOP_FORUM_NAME'],
	'U_VIEW_FORUM'	=> append_sid("shop.$phpEx", "mode=cat"),
));

$page_title = $user->lang['SHOP_FORUM_NAME'];

//Check if you are locked or not
if (!$auth->acl_get('u_shop_mod'))
{
    trigger_error('NOT_AUTHORISED');
}

// Disable shop, if the install file is still present in the root folder
if (file_exists($phpbb_root_path . 'install_shop.php'))
{
	// Adjust the message slightly according to the permissions
	if ( $auth->acl_gets('a_') )
	{
		$message = $user->lang['SHOP_REMOVE_INSTALL'];
	}
	else
	{
		$message = $shop_config['disbale_message'];
	}
	trigger_error($message);
}

// Shop Version
$version = $config['shop_mod_version'];
$mode = request_var('mode', '');

// Is Shop enable?
if ($config['shop_enable'] && $shop->cash->money_on && $config['shop_mod_version'])
{
	$shop_open = true;
}
else
{
    meta_refresh(5, append_sid($phpbb_root_path . 'index.' . $phpEx));
	trigger_error($shop_config['disable_message']);
}

$template->assign_vars(array(
	'S_OPEN'		=> $shop_open,
	'S_STOCK'		=> true,
	'SHOP_VERS'		=> $version,
	'SHOP_YOU_HAVE'	=> sprintf($user->lang['SHOP_YOU_HAVE'], number_format_points($user->data['user_points']), $config['points_name']),
	'U_USER_LIST'	=> append_sid("{$phpbb_root_path}shop.$phpEx", 'mode=inventory_list'),
));

switch ($mode)
{
	case 'cat':
		// Generate the navigation
		$cat_id = 0;
		$shop->generate_cat_list($cat_id);
		$template->assign_vars(array(
			'S_OPEN'		=> false,
			'S_CATS'		=> true,
			'SHOP_VERS'		=> $version,
			'SHOP_YOU_HAVE'	=> sprintf($user->lang['SHOP_YOU_HAVE'], number_format_points($user->data['user_points']), $config['points_name']),
		));
	break;

	case 'showcat':
		$cat_id = request_var('c', 0);

		$cat_data = $shop->get_cat_info($cat_id);
		$shop->generate_cat_nav($cat_data);
		$shop->generate_cat_list($cat_id);

		$template->assign_vars(array(
			'S_OPEN'		=> true,
			'S_CATS'		=> true,
			'SHOP_VERS'		=> $version,
			'SHOP_YOU_HAVE'	=> sprintf($user->lang['SHOP_YOU_HAVE'], number_format_points($user->data['user_points']), $config['points_name']),
		));

		if ($shop_open)
		{
			if (!$user->data['is_registered'])
			{
				$can_buy = false;
			}

			// Pagination Items
			$start			= request_var('start', 0);
			$number			= $shop_config['items_per_page'];

			$sql = 'SELECT COUNT(item) AS total_items
				FROM ' . SHOP_ITEMS_TABLE . '
				WHERE item > 0
					AND shop_cat_id = ' . (int) $cat_id;
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$total_items = $row['total_items'];
			$db->sql_freeresult($result);

			$sql = 'SELECT *
				FROM ' . SHOP_ITEMS_TABLE . '
				WHERE shop_cat_id = ' . (int)$cat_id;
			$result = $db->sql_query_limit($sql, $number, $start);

			while ($row = $db->sql_fetchrow($result))
			{
				$icon_show_width = $icon_show_height = 0;
				$insuf_funds	= $row['price'] > $user->data['user_points'] ? true : false;

				// Define image size and scale ratio
				// Code partly from Michael Bailey
				$orig_width = $row['icon_width'];
				$orig_height = $row['icon_height'];
				$scale = min($shop_config['shop_icon_width']/$orig_width, $shop_config['shop_icon_height']/$orig_height);

				// If the image is larger than the max shrink it
				if ($scale < 1)
				{
					$icon_show_width = floor($scale*$orig_width);
					$icon_show_height = floor($scale*$orig_height);
				}
				else
				{
					$icon_show_width = $orig_width;
					$icon_show_height = $orig_height;
				}

				// Check if Highslide JS is installed
				if (file_exists($phpbb_root_path . 'highslide/highslide-full.js'))
				{
					$show_icon = '<a href="' . $phpbb_root_path . 'images/shop_icons/' . $row['icon'] . '" class="highslide" onclick="return hs.expand(this)" title="' . $user->lang['SHOP_HIGHSLIDE_CLICK'] . '"><img src="' . $phpbb_root_path . 'images/shop_icons/' . $row['icon'] . '" width="' . $icon_show_width . '" height="' . $icon_show_height . '" alt="' . $user->lang['SHOP_HIGHSLIDE_CLICK'] . '" /></a>';
				}
				else
				{
					$show_icon = '<a href="' . $phpbb_root_path . 'images/shop_icons/' . $row['icon'] . '" onclick=" window.open(this.href); return false" title="' . $user->lang['SHOP_HIGHSLIDE_CLICK'] . '"><img src="' . $phpbb_root_path . 'images/shop_icons/' . $row['icon'] . '" width="' . $icon_show_width . '" height="' . $icon_show_height . '" alt="' . $user->lang['SHOP_HIGHSLIDE_CLICK'] . '" /></a>';
				}

				$template->assign_block_vars('item_list', array(
					'NAME'			=> $row['name'],
					'DESCRIPTION'	=> $row['description'],
					'ICON'			=> $show_icon,
					'PRICE'			=> $shop->cash->format_cash_string($row['price']),
					'QUANTITY'		=> $row['quantity'],
					'INSUFFICENT'	=> sprintf($user->lang['SHOP_INSUFFICIENT_FUNDS'], $config['points_name']),
					'S_INSTANT'		=> $row['instant_use'],
					'S_INSUF_FUNDS'	=> $insuf_funds,
					'U_BUY'			=> append_sid("{$phpbb_root_path}shop.$phpEx", 'mode=buy&amp;item=' . $row['item'] .'&amp;c=' . $cat_id),
				));
			}

			$template->assign_vars(array(
				'PAGINATION' 	=> generate_pagination(append_sid("{$phpbb_root_path}shop.$phpEx", "mode=showcat&amp;c=" . $cat_id), $total_items, $number, $start, true),
				'PAGE_NUMBER'	=> on_page($total_items, $number, $start),
				'TOTAL_ITEMS'	=> ($total_items == 1) ? $user->lang['SHOP_TOTAL_ITEM'] : sprintf($user->lang['SHOP_TOTAL_ITEMS'], $total_items),
			));
		}
	break;

	case 'buy';

		//first of all, we make sure the item exists
		$item_id = request_var('item', 0);
		$cat_id = request_var('c', 0);
		$number = '';
		$start = request_var('start', 0);

		if (!$item = $shop->get_item($item_id))
		{
			trigger_error('SHOP_ITEM_ID_NOT_FOUND');
		}

		if (confirm_box(true))
		{
			$item->buy($cat_id);
		}
		else
		{
			$s_hidden_fields = build_hidden_fields(array(
				'submit'	=> true,
				)
			);

			$confirm_string = sprintf($user->lang['SHOP_ITEM_CONFIRM_BUY'], $item->data['name'], $shop->cash->format_cash_string($item->data['price']), $config['points_name']);
			confirm_box(false, $confirm_string, $s_hidden_fields);
		}

	break;

	case 'buy_from_user';

		$item_id = request_var('item', 0);
		$user_id = request_var('u', 0);
		$buyer = request_var('b', 0);
		// Coming from explanation
		// 1 = from memberlist
		// 2 = from user selling page
		$coming_from = request_var('f', 0);

		//first of all, we make sure the item exists
		$sql = 'SELECT ui.*, i.*
			FROM ' . SHOP_ITEMS_USER_TABLE . ' ui
			LEFT JOIN ' . SHOP_ITEMS_TABLE . ' i
				ON ui.item = i.item
			WHERE ui.item = ' . (int) $item_id . '
				AND ui.user = ' . (int) $user_id;
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$item = $shop->get_item($item_id, $user_id))
		{
			trigger_error('SHOP_ITEM_ID_NOT_FOUND');
		}

		if (confirm_box(true))
		{
			$item->buy_from_user($item_id, $user_id, $buyer, $coming_from);
		}
		else
		{
		$s_hidden_fields = build_hidden_fields(array(
			'submit'	=> true,
			)
		);

			$confirm_string = sprintf($user->lang['SHOP_ITEM_CONFIRM_BUY'], $row['name'], $shop->cash->format_cash_string($row['sell_to_price']), $config['points_name']);
			confirm_box(false, $confirm_string, $s_hidden_fields);
		}
	break;

	case 'inventory_list':

		//Template variables for the navigation
		$template->assign_block_vars('navlinks', array(
			'FORUM_NAME'	=> $user->lang['SHOP_USER_SELL'],
			'U_VIEW_FORUM'	=> append_sid("shop.$phpEx", "mode=inventory_list"),
		));

		$item_id = request_var('item', 0);
		$user_id = request_var('u', 0);
		$buyer = request_var('b', 0);

		// Pagination Items
		$start			= request_var('start', 0);
		$number			= $shop_config['items_per_page'];

		$sql = 'SELECT COUNT(item) AS total_items
			FROM ' . SHOP_ITEMS_USER_TABLE . '
			WHERE sell_to_price > 0
				AND user <> ' . $user->data['user_id'];
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$total_items = $row['total_items'];
		$db->sql_freeresult($result);

		// Select all user item, which have a sell price
		$sql = 'SELECT i.*, s.*, u.*
			FROM ' . SHOP_ITEMS_TABLE . ' i
			LEFT JOIN ' . SHOP_ITEMS_USER_TABLE . ' s
				ON i.item = s.item
			LEFT JOIN ' . USERS_TABLE . ' u
				ON s.user = u.user_id
			WHERE s.sell_to_price > 0
				AND s.user <> ' . $user->data['user_id'] . '
			ORDER BY s.user ASC';
		$result = $db->sql_query_limit($sql, $number, $start);

		while ($row = $db->sql_fetchrow($result))
		{
			$icon_show_width = $icon_show_height = 0;
			$insuf_funds	= $row['sell_to_price'] > $user->data['user_points'] ? true : false;

			// Define image size and scale ratio
			// Code partly from Michael Bailey
			$orig_width = $row['icon_width'];
			$orig_height = $row['icon_height'];
			$scale = min($shop_config['shop_icon_width']/$orig_width, $shop_config['shop_icon_height']/$orig_height);

			// If the image is larger than the max shrink it
			if ($scale < 1)
			{
				$icon_show_width = floor($scale*$orig_width);
				$icon_show_height = floor($scale*$orig_height);
			}
			else
			{
				$icon_show_width = $orig_width;
				$icon_show_height = $orig_height;
			}

			// Check if Highslide JS is installed
			if (file_exists($phpbb_root_path . 'highslide/highslide-full.js'))
			{
				$show_icon = '<a href="' . $phpbb_root_path . 'images/shop_icons/' . $row['icon'] . '" class="highslide" onclick="return hs.expand(this)" title="' . $user->lang['SHOP_HIGHSLIDE_CLICK'] . '"><img src="' . $phpbb_root_path . 'images/shop_icons/' . $row['icon'] . '" width="' . $icon_show_width . '" height="' . $icon_show_height . '" alt="' . $user->lang['SHOP_HIGHSLIDE_CLICK'] . '" /></a>';
			}
			else
			{
				$show_icon = '<a href="' . $phpbb_root_path . 'images/shop_icons/' . $row['icon'] . '" onclick=" window.open(this.href); return false" title="' . $user->lang['SHOP_HIGHSLIDE_CLICK'] . '"><img src="' . $phpbb_root_path . 'images/shop_icons/' . $row['icon'] . '" width="' . $icon_show_width . '" height="' . $icon_show_height . '" alt="' . $user->lang['SHOP_HIGHSLIDE_CLICK'] . '" /></a>';
			}

			$template->assign_block_vars('item_user_list', array(
				'S_ITEM_USER_LIST'	=> true,
				'NAME'				=> $row['name'],
				'DESCRIPTION'		=> $row['description'],
				'ICON'				=> $show_icon,
				'PRICE'				=> $shop->cash->format_cash_string($row['sell_to_price']),
				'QUANTITY'			=> $row['quantity_user'],
				'USER_NAME'			=> get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']),
				'INSUFFICENT'		=> sprintf($user->lang['SHOP_INSUFFICIENT_FUNDS'], $config['points_name']),
				'S_INSUF_FUNDS'		=> $insuf_funds,
				'U_BUY_FROM_USER'	=> append_sid("{$phpbb_root_path}shop.$phpEx", "mode=buy_from_user&amp;item=" . $row['item'] . "&amp;u=" . $row['user_id'] . "&amp;b=" . $user->data['user_id'] . "&amp;f=2"),
			));
		}

		$template->assign_vars(array(
			'S_ITEM_USER_LIST'	=> true,
			'BACK'				=> append_sid("{$phpbb_root_path}shop.$phpEx", "mode=cat"),
			'PAGINATION' 		=> generate_pagination(append_sid("{$phpbb_root_path}shop.$phpEx", "mode=inventory_list"), $total_items, $number, $start, true),
			'PAGE_NUMBER'		=> on_page($total_items, $number, $start),
			'TOTAL_ITEMS'		=> ($total_items == 1) ? $user->lang['SHOP_TOTAL_ITEM'] : sprintf($user->lang['SHOP_TOTAL_ITEMS'], $total_items),
		));

	break;
}

page_header($page_title);

page_footer();

?>